@@ -1040,7 +1040,7 @@ typedef enum
 {
     GLOBUS_GFS_LOG_ERR = 0x01,
     GLOBUS_GFS_LOG_WARN = 0x02,
-    GLOBUS_GFS_LOG_STATUS = 0x04,
+    GLOBUS_GFS_LOG_TRANSFER = 0x04,
     GLOBUS_GFS_LOG_INFO = 0x08,
     GLOBUS_GFS_LOG_DUMP = 0x10,
     
