@@ -43,8 +43,6 @@ GlobusDebugDeclare(GLOBUS_GRIDFTP_SERVER_FILE);
 
 GlobusDebugDefine(GLOBUS_GRIDFTP_SERVER_FILE);
 
-#define GLOBUS_L_GFS_FILE_CKSM_BS 1024*1024
-
 typedef void
 (*globus_l_gfs_file_cksm_cb_t)(
     globus_result_t                     result,
@@ -405,6 +403,26 @@ globus_l_gfs_file_cksm_verify(
     GlobusGFSFileDebugExit();
 }
 
+static 
+globus_bool_t
+globus_l_gfs_file_timeout_cb(
+    globus_xio_handle_t                 handle,
+    globus_xio_operation_type_t         type,
+    void *                              user_arg)
+{
+    GlobusGFSName(globus_l_gfs_file_timeout_cb);
+
+    GlobusGFSFileDebugEnter();
+
+    globus_gfs_log_message(
+        GLOBUS_GFS_LOG_WARN,
+        "A file access timeout has occurred.\n");
+   
+    GlobusGFSFileDebugExit();
+
+    return GLOBUS_TRUE;
+}
+
 static
 void
 globus_l_gfs_file_close_cb(
@@ -731,7 +749,13 @@ globus_l_gfs_file_stat(
             goto error_alloc2;
         }
         
-        snprintf(dir_path, sizeof(dir_path), "%s/%s", basepath, filename);
+        snprintf(
+            dir_path, 
+            sizeof(dir_path), 
+            "%s/%s", 
+            (basepath[0] != '/' || basepath[1] != '\0') ? basepath : "", 
+            filename);
+            
         dir_path[MAXPATHLEN - 1] = '\0';
         
         for(i = 0;
@@ -1186,7 +1210,7 @@ globus_l_gfs_file_cksm_read_cb(
 
 error_register:
 error_read:
-    globus_xio_close(handle, NULL);
+    globus_xio_register_close(handle, NULL, NULL, NULL);
     handle = NULL;
     globus_gridftp_server_finished_command(monitor->op, result, NULL);    
     globus_free(monitor);
@@ -1266,7 +1290,7 @@ globus_l_gfs_file_open_cksm_cb(
 error_register:
 error_seek:
 error_open:
-    globus_xio_close(handle, NULL);
+    globus_xio_register_close(handle, NULL, NULL, NULL);
     handle = NULL;
     if(monitor->internal_cb)
     {
@@ -1299,6 +1323,7 @@ globus_l_gfs_file_cksm(
     globus_xio_handle_t                 file_handle;
     globus_l_gfs_file_cksm_monitor_t *  monitor;
     globus_size_t                       block_size;
+    int                                 timeout;
     GlobusGFSName(globus_l_gfs_file_cksm);
     GlobusGFSFileDebugEnter();
     
@@ -1325,6 +1350,28 @@ globus_l_gfs_file_cksm(
         result = GlobusGFSErrorWrapFailed("globus_xio_attr_init", result);
         goto error_cntl;
     }
+
+    timeout = globus_gfs_config_get_int("file_timeout");
+    if(timeout > 0)
+    {
+        globus_reltime_t                delay;
+
+        GlobusTimeReltimeSet(delay, timeout, 0);
+        result = globus_xio_attr_cntl(
+            attr,
+            NULL,
+            GLOBUS_XIO_ATTR_SET_TIMEOUT_ALL,
+            globus_l_gfs_file_timeout_cb,
+            &delay,
+            NULL);
+        if(result != GLOBUS_SUCCESS)
+        {
+            globus_gfs_log_message(
+                GLOBUS_GFS_LOG_WARN,
+                "Unable to set file access timeout of %d seconds\n", 
+                timeout);
+        }
+    }
     
     result = globus_xio_stack_init(&stack, NULL);
     if(result != GLOBUS_SUCCESS)
@@ -1348,7 +1395,7 @@ globus_l_gfs_file_cksm(
         goto error_create;
     }
 
-    block_size = GLOBUS_L_GFS_FILE_CKSM_BS;
+    globus_gridftp_server_get_block_size(op, &block_size);
 
     monitor = (globus_l_gfs_file_cksm_monitor_t *) globus_calloc(
         1, sizeof(globus_l_gfs_file_cksm_monitor_t) + block_size);
@@ -1376,7 +1423,7 @@ globus_l_gfs_file_cksm(
         result = GlobusGFSErrorWrapFailed("globus_xio_register_open", result);
         goto error_register;
     }
-    
+
     globus_xio_attr_destroy(attr);
     globus_xio_stack_destroy(stack);
     
@@ -1384,7 +1431,7 @@ globus_l_gfs_file_cksm(
     return GLOBUS_SUCCESS;
 
 error_register:
-    globus_xio_close(file_handle, NULL);
+    globus_xio_register_close(file_handle, NULL, NULL, NULL);
     file_handle = NULL;
     globus_free(monitor);
     
@@ -1840,7 +1887,7 @@ globus_l_gfs_file_open_write_cb(
     }
 
     globus_gridftp_server_begin_transfer(
-        monitor->op, 0, monitor);
+        monitor->op, GLOBUS_GFS_EVENT_TRANSFER_ABORT, monitor);
     
     globus_mutex_lock(&monitor->lock);
     {
@@ -1914,6 +1961,8 @@ globus_l_gfs_file_open(
     globus_result_t                     result;
     globus_xio_attr_t                   attr;
     globus_xio_stack_t                  stack;
+    char *                              perms;
+    int                                 timeout;
     GlobusGFSName(globus_l_gfs_file_open);
     GlobusGFSFileDebugEnter();
 
@@ -1926,9 +1975,6 @@ globus_l_gfs_file_open(
         goto error_attr;
     }
     
-    /* XXX should probably have an option to specify create mode.
-     * for now, just the default (u+rw)
-     */     
     result = globus_xio_attr_cntl(
         attr,
         globus_l_gfs_file_driver,
@@ -1939,6 +1985,61 @@ globus_l_gfs_file_open(
         result = GlobusGFSErrorWrapFailed("globus_xio_attr_init", result);
         goto error_cntl;
     }
+
+    if(open_flags & GLOBUS_XIO_FILE_CREAT)
+    {
+        perms = globus_gfs_config_get_string("perms");
+        if(perms != NULL)
+        {
+            int                             p = 0;
+            
+            p = strtoul(perms, NULL, 8);
+            if(p > 0 || 
+                (perms[0] == '0' && perms[1] == '\0'))
+            {
+                result = globus_xio_attr_cntl(
+                    attr,
+                    globus_l_gfs_file_driver,
+                    GLOBUS_XIO_FILE_SET_MODE,
+                    p);
+                if(result != GLOBUS_SUCCESS)
+                {
+                    globus_gfs_log_message(
+                        GLOBUS_GFS_LOG_WARN,
+                        "Failed to set default permissions to: %s\n", perms);
+                }
+            }
+            else
+            {
+                globus_gfs_log_message(
+                    GLOBUS_GFS_LOG_WARN,
+                    "Invalid default permissions: %s\n", perms);
+            }
+        }
+    }
+
+    timeout = globus_gfs_config_get_int("file_timeout");
+    if(timeout > 0)
+    {
+        globus_reltime_t                delay;
+
+        GlobusTimeReltimeSet(delay, timeout, 0);
+        result = globus_xio_attr_cntl(
+            attr,
+            NULL,
+            GLOBUS_XIO_ATTR_SET_TIMEOUT_ALL,
+            globus_l_gfs_file_timeout_cb,
+            &delay,
+            NULL);
+        if(result != GLOBUS_SUCCESS)
+        {
+            globus_gfs_log_message(
+                GLOBUS_GFS_LOG_WARN,
+                "Unable to set file access timeout of %d seconds\n", 
+                timeout);
+        }
+    }
+    
     
     result = globus_xio_stack_init(&stack, NULL);
     if(result != GLOBUS_SUCCESS)
@@ -1974,6 +2075,7 @@ globus_l_gfs_file_open(
         goto error_register;
     }
     
+    
     globus_xio_attr_destroy(attr);
     globus_xio_stack_destroy(stack);
     
@@ -1981,7 +2083,7 @@ globus_l_gfs_file_open(
     return GLOBUS_SUCCESS;
 
 error_register:
-    globus_xio_close(*file_handle, NULL);
+    globus_xio_register_close(*file_handle, NULL, NULL, NULL);
     *file_handle = NULL;
 error_create:
 error_push:
@@ -2387,7 +2489,7 @@ globus_l_gfs_file_open_read_cb(
     }
     
     globus_gridftp_server_begin_transfer(
-        monitor->op, 0, monitor);
+        monitor->op, GLOBUS_GFS_EVENT_TRANSFER_ABORT, monitor);
     
     globus_mutex_lock(&monitor->lock);
     monitor->first_read = GLOBUS_TRUE;
@@ -2500,15 +2602,17 @@ globus_l_gfs_file_event(
     switch(event_info->type)
     {
         case GLOBUS_GFS_EVENT_TRANSFER_ABORT:
-            /* currently this will just prevent any further reads
-              or writes from being registered.  should probably
-              cancel/flush pending/current reads and writes somehow */
             globus_mutex_lock(&monitor->lock);
             {
                 monitor->aborted = GLOBUS_TRUE;
             }
             globus_mutex_unlock(&monitor->lock);
-            fprintf(stderr, "globus_l_gfs_file_event: aborted.\n");
+            
+            globus_xio_handle_cancel_operations(
+                monitor->file_handle,
+                GLOBUS_XIO_CANCEL_OPEN | 
+                GLOBUS_XIO_CANCEL_READ |
+                GLOBUS_XIO_CANCEL_WRITE);
             break;
             
         default:
@@ -2548,23 +2652,24 @@ globus_l_gfs_file_destroy(
     void *                              user_arg)
 {
     gfs_l_file_session_t *              session_h;
-/*
     session_h = (gfs_l_file_session_t *) user_arg;
 
-    if(session_h->sbj != NULL)
+    if(session_h)
     {
-        globus_free(session_h->sbj);
+        if(session_h->sbj != NULL)
+        {
+            globus_free(session_h->sbj);
+        }
+        if(session_h->username != NULL)
+        {
+            globus_free(session_h->username);
+        }
+        if(session_h->pw != NULL)
+        {
+            globus_free(session_h->pw);
+        }
+        globus_free(session_h);
     }
-    if(session_h->username != NULL)
-    {
-        globus_free(session_h->sbj);
-    }
-    if(session_h->pw != NULL)
-    {
-        globus_free(session_h->sbj);
-    }
-    globus_free(session_h);
-*/
 }
 
 static
